/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
/* Olaf Leidinger <oleid@mescharet.de>  20130213  */

const Gdk = imports.gi.Gdk;
const Gio = imports.gi.Gio;
const GLib = imports.gi.GLib;
const GnomeDesktop = imports.gi.GnomeDesktop;
const Lang = imports.lang;
const Mainloop = imports.mainloop; // timer
const Shell = imports.gi.Shell;
const St = imports.gi.St;
const Clutter = imports.gi.Clutter;

const Main        = imports.ui.main;
const MessageTray = imports.ui.messageTray;
const PanelMenu   = imports.ui.panelMenu;
const PopupMenu   = imports.ui.popupMenu;
const Panel       = imports.ui.panel;

const Gettext = imports.gettext.domain('gnome-shell-extensions');
const _ = Gettext.gettext;
const N_ = function(e) { return e; };

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();

// TODO: make this configurable via gsettings
const defaultTeaList = [
            {name : "Green tea", time : 180 }, 
            {name : "Black tea", time : 210 }, 
            {name : "Fruit tea", time : 7 * 60}, 
            {name : "White tea", time : 120} 
            ];

const TeaTime = new Lang.Class({
    Name : 'TeaTime',
    Extends : PanelMenu.Button,

    _init : function() {
        this.parent(0.0, "TeaTime");

        this._logo = new St.Icon({
            icon_name : 'utilities-teatime',
            style_class : 'system-status-icon'
        });

        // set timer widget

        this._timer = new St.DrawingArea({
            reactive : true
        });
        this._timer.set_width(20);
        this._timer.set_height(20);
        this._timer.connect('repaint', Lang.bind(this, this._drawTimer));

        this.actor.add_actor(this._logo);

        this._dt = 4;

        this._createMenu();
    },

    _createMenu : function() {
        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
        this._addTeaList();
    },
    _formatTime : function(seconds) {
        // from
        // https://github.com/kennym/gnome-shell-extension-timer/blob/master/timer%40olebowle.gmx.com/extension.js
        let  hours = parseInt(seconds / 3600);
        seconds -= hours * 3600;
        
        let  minutes = parseInt(seconds / 60);
        seconds -= minutes * 60;

        // Weird way to show 2-digit number, but js doesn't have a native
        // padding function
        if (hours < 10)
            hours = "0" + hours.toString();
        else
            hours = hours.toString();

        if (minutes < 10)
            minutes = "0" + minutes.toString();
        else
            minutes = minutes.toString();

        if (seconds < 10)
            seconds = "0" + seconds.toString();
        else
            seconds = seconds.toString();

        if (hours != "00")
            return hours + ":" + minutes + ":" + seconds;
        else
            return minutes + ":" + seconds;
    },
    _addTeaList : function(config, output) {
        let  item = new PopupMenu.PopupMenuItem(_("brewing times"));
        item.label.add_style_class_name('display-subtitle');
        item.actor.reactive = false;
        item.actor.can_focus = false;
        this.menu.addMenuItem(item);
        this._callbacks = [];

        defaultTeaList.sort(function(a, b) {
            return -1 * (a.time < b.time) + (a.time > b.time);
        });
        
        for ( var i = 0; i < defaultTeaList.length; i++) {
            let tea  = defaultTeaList[i];
            let item = new PopupMenu.PopupMenuItem(this._formatTime(tea.time) + " - " + tea.name);

            this._callbacks.push( function() {this._initCountdown(tea.time); });
            item.connect('activate', Lang.bind(this, this._callbacks[i]));
            this.menu.addMenuItem(item);
        }
    },
    _showNotification : function(subject, text) {
        let source = new MessageTray.Source("TeaTime applet", 'utilities-teatime');
        Main.messageTray.add(source);
        
        let notification = new MessageTray.Notification(source, subject, text);
        notification.setTransient(true);
        source.notify(notification);
    },
    _initCountdown : function(time) {
        this._countdownMax = time;
        this._timeRemaining = time;
        this._progress = 0;
        this.actor.remove_actor(this._logo);
        this.actor.add_actor(this._timer);
        this._showNotification("Timer set!", time + "s to go");
        Mainloop.timeout_add_seconds(this._dt, Lang.bind(this, this._doCountdown));
    },
    _doCountdown : function() {
        this._timeRemaining -= this._dt;
        this._progress       = (this._countdownMax - this._timeRemaining) / this._countdownMax;

        if (this._timeRemaining <= 0) {
            this.actor.remove_actor(this._timer);
            this.actor.add_actor(this._logo);
            this._showNotification("Your tea is ready!",
                    "Drink it, while it is hot!");
        } else {
            Mainloop.timeout_add_seconds(this._dt, Lang.bind(this, this._doCountdown));
            this._timer.queue_repaint();
        }
    },
    _drawTimer : function() {
        let[width, height] = this._timer.get_surface_size();
        let cr = this._timer.get_context();
        let pi = Math.PI;
        let  r = Math.min(width, height) * 0.5;;

        // TODO: get colors from current theme!
        cr.setSourceRGB(0, 0, 0);
        cr.rectangle(0, 0, width, height);
        cr.fill();

        cr.translate(Math.floor(width / 2), Math.floor(height / 2));
        cr.save();

        cr.setSourceRGB(0.2, 0.2, 0.2);
        cr.moveTo(0, 0);
        cr.arc(0, 0, r, 3 / 2 * pi + 2 * pi * this._progress, 3 / 2 * pi + 2
                * pi);
        cr.fill();

        cr.setSourceRGB(0.8, 0.8, 0.8);
        cr.moveTo(0, 0);
        cr.arc(0, 0, r, 3 / 2 * pi, 3 / 2 * pi + 2 * pi * this._progress);
        cr.fill();
    }
});

function init(metadata) {
    // TODO: at some point, add translations
    ;
}

let _TeaTime;

function enable() {
    _TeaTime = new TeaTime();
    Main.panel.addToStatusArea('teatime', _TeaTime);
}

function disable() {
    _TeaTime.destroy();
};
